@@ -26,9 +26,24 @@ def _only_concrete_models(iterable):
             yield model
 
 
+class InvalidSpec(Exception):
+    pass
+
+
+_valid_keys = {"model", "filter", "save_as_new", "delete_missing"}
+
+
+def _validate_spec(spec):
+    if "model" not in spec:
+        raise InvalidSpec(f"The spec {spec!r} requires a 'model' key")
+    if unknown := (set(spec.keys()) - _valid_keys):
+        raise InvalidSpec(f"The spec {spec!r} contains unknown keys: {unknown!r}")
+    return spec
+
+
 def specs_for_models(models, spec=None):
     spec = {} if spec is None else spec
-    return (spec | {"model": cls._meta.label_lower} for cls in models)
+    return (_validate_spec(spec | {"model": cls._meta.label_lower}) for cls in models)
 
 
 def specs_for_derived_models(cls, spec=None):
@@ -80,42 +95,51 @@ def load_dump(data, *, progress=silence, ignorenonexistent=False):
 
     progress(f"Loaded {len(data['objects'])} objects")
 
-    force_insert_pk_map = defaultdict(dict)
-    force_insert_models = {
-        spec["model"] for spec in data["specs"] if spec.get("force_insert")
+    save_as_new_pk_map = defaultdict(dict)
+    save_as_new_models = {
+        spec["model"] for spec in data["specs"] if spec.get("save_as_new")
     }
 
     with transaction.atomic():
         for spec in data["specs"]:
             if objs := objects[spec["model"]]:
                 for ds in objs:
-                    _do_save(ds, pk_map=force_insert_pk_map, models=force_insert_models)
+                    _do_save(
+                        ds,
+                        pk_map=save_as_new_pk_map,
+                        save_as_new_models=save_as_new_models,
+                    )
                     seen_pks[ds.object._meta.label_lower].add(ds.object.pk)
 
             progress(f"Saved {len(objs)} {spec['model']} objects")
 
         for spec in data["specs"]:
+            if not spec.get("delete_missing"):
+                continue
+
             queryset = _model_queryset(spec)
-            if deleted := queryset.exclude(pk__in=seen_pks[spec["model"]]).delete():
+            deleted = queryset.exclude(pk__in=seen_pks[spec["model"]]).delete()
+            if deleted[0]:
                 progress(f"Deleted {spec['model']} objects: {deleted}")
 
 
-def _do_save(ds, *, pk_map, models):
-    if ds.object._meta.label_lower in models:
-        # Map old PKs to new
-        for f in ds.object._meta.get_fields():
-            if (
-                f.concrete
-                and f.related_model
-                and f.related_model._meta.label_lower in models
-            ):
-                if getattr(ds.object, f.column) in pk_map[f.related_model]:
-                    setattr(
-                        ds.object,
-                        f.name,
-                        pk_map[f.related_model][getattr(ds.object, f.column)],
-                    )
+def _do_save(ds, *, pk_map, save_as_new_models):
+    # Map old PKs to new
+    for f in ds.object._meta.get_fields():
+        if (
+            f.concrete
+            and f.related_model
+            and f.related_model._meta.label_lower in save_as_new_models
+        ):
+            # XXX Do this unconditionally until we find a reason
+            # if getattr(ds.object, f.column) in pk_map[f.related_model]:
+            setattr(
+                ds.object,
+                f.name,
+                pk_map[f.related_model][getattr(ds.object, f.column)],
+            )
 
+    if ds.object._meta.label_lower in save_as_new_models:
         # Do the saving
         old_pk = ds.object.pk
         ds.object.pk = None
